package com.urbandroid.sleep.captchapack;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.*;
import android.widget.TextView;
import com.urbandroid.sleep.captcha.CaptchaSupport;
import com.urbandroid.sleep.captcha.CaptchaSupportFactory;
import com.urbandroid.sleep.captcha.RemainingTimeListener;
import com.urbandroid.sleep.captchapack.util.DateUtil;

public abstract class BaseCaptcha extends AppCompatActivity {

    private static final long ALIVE_TIMEOUT = 60000;

    private TextView timeoutText;

    private CaptchaSupport captchaSupport;

    private MenuItem addTimeAction;

    private String originalTitle;

    public static final String TAG = "SleepCaptchaPack";

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        captchaSupport = CaptchaSupportFactory.create(this, intent);
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        originalTitle = getTitle().toString();

        captchaSupport = CaptchaSupportFactory.create(this);
        if (captchaSupport.isOperationalMode()) {
            timeoutText = createTimoutTextView();
            alive();
            captchaSupport.setRemainingTimeListener(new RemainingTimeListener() {
                @Override
                public void timeRemain(int seconds, int aliveTimeout) {

                    if (timeoutText != null) {
                        if (seconds > 0) {
                            timeoutText.setVisibility(View.VISIBLE);
                            timeoutText.setText(DateUtil.formatSeconds((int) seconds));
                            timeoutText.setCompoundDrawablesWithIntrinsicBounds(R.drawable.silent, 0, 0, 0);
                            if (addTimeAction != null) {
                                addTimeAction.setVisible(true);
                            }
                        } else {
                            timeoutText.setVisibility(View.GONE);
                        }
                    }
                }
            });
        }
    }

    public TextView createTimoutTextView() {
        if (getSupportActionBar() != null) {
            LayoutInflater inflator = (LayoutInflater) this .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            TextView timeoutText = (TextView) inflator.inflate(R.layout.base_captcha_alive_timeout, (ViewGroup) findViewById(R.id.toolbar), false);
            ActionBar.LayoutParams lp = new ActionBar.LayoutParams(android.app.ActionBar.LayoutParams.WRAP_CONTENT, android.app.ActionBar.LayoutParams.MATCH_PARENT, Gravity.RIGHT | Gravity.CENTER_VERTICAL);
            getSupportActionBar().setCustomView(timeoutText, lp);
            getSupportActionBar().setDisplayShowCustomEnabled(true);
            return timeoutText;
        }
        return null;
    }

    public void alive() {
        getCaptchaSupport().alive();
    }

    public void solved() {
        getCaptchaSupport().solved();
    }

    public void unsolved() {
        getCaptchaSupport().unsolved();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.captcha_menu, menu);

        addTimeAction = menu.findItem(R.id.menu_item_add_time);
        if (addTimeAction != null) {
            addTimeAction.setVisible(false);
            addTimeAction.setTitle(getString(R.string.add_time, ALIVE_TIMEOUT / 60000));
        }

        return super.onCreateOptionsMenu(menu);
    }

    public void updateProgress(int progress, int max) {
        if (getSupportActionBar() != null) {
            getSupportActionBar().setTitle(progress + " / " + max + " " + originalTitle);
        }
    }

    @Override
    public void onUserInteraction() {
        alive();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_item_add_time:
                alive();
                return true;
        }
        return true;
    }


    public CaptchaSupport getCaptchaSupport() {
        return captchaSupport;
    }

    @Override
    protected void onStart() {
        super.onStart();
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        getCaptchaSupport().destroy();
    }
}
