package com.urbandroid.sleep.captchapack.cc;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;

public class CaptchaCaptcha extends BaseCaptcha {

    private int solvedCounter = 0;

    private final String EXTRA_SOLVED_COUNTER = "extra_solved_counter";

    private int difficulty = 5;

    private Captcha captcha;

    private ImageView imageView;


    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);

        outState.putInt(EXTRA_SOLVED_COUNTER, solvedCounter);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_captcha_captcha);

        if (savedInstanceState != null) {
            solvedCounter = savedInstanceState.getInt(EXTRA_SOLVED_COUNTER, 0);
        }

        difficulty = getCaptchaSupport().getDifficulty();

        imageView = (ImageView) findViewById(R.id.image);
        final EditText edit = (EditText) findViewById(R.id.edit);
        findViewById(R.id.refresh).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                captcha = generateCaptcha(imageView, difficulty);
                edit.setText("");
            }
        });

        captcha = generateCaptcha(imageView, difficulty);

        edit.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                Log.i(TAG, " answer " + s.toString() + " ??? " + captcha.answer);
                if (captcha.checkAnswer(s.toString())) {
                    solvedCounter++;
                    updateProgress(solvedCounter, difficulty);
                    if (solvedCounter >= difficulty) {
                        solved();
                        finish();
                        return;
                    }
                    captcha = generateCaptcha(imageView, difficulty);
                    edit.setText("");
                } else {
                    alive();
                }
            }
        });
        edit.setText("");

        updateProgress(solvedCounter + 1, difficulty);
    }

    private Captcha generateCaptcha(ImageView img, int difficulty) {
        final Captcha c = new TextCaptcha(2 + difficulty, TextCaptcha.TextOptions.NUMBERS_AND_LETTERS);
        img.setImageBitmap(c.image);
        return c;
    }


    @Override
    public void onBackPressed() {
        super.onBackPressed();
        unsolved();
    }
}
