package com.urbandroid.sleep.captchapack.context;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Log;
import com.urbandroid.sleep.captchapack.BaseCaptcha;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author petr
 */
public class Settings implements SettingKeys {

    private Context context;

    public Settings(Context context) {
        this.context = context;
    }


    /** Helper methods for properties **/
    public void save(String key, int value) {
        SharedPreferences.Editor edit = getDefaultSharedPreferences(context).edit();
        edit.putInt(key, value);
        edit.commit();
    }

    public void save(String key, String value) {
        SharedPreferences.Editor edit = getDefaultSharedPreferences(context).edit();
        edit.putString(key, value);
        edit.commit();
    }

    public void save(String key, boolean value) {
        SharedPreferences.Editor edit = getDefaultSharedPreferences(context).edit();
        edit.putBoolean(key, value);
        edit.commit();
    }

    private SharedPreferences getDefaultSharedPreferences(Context context) {
        return PreferenceManager.getDefaultSharedPreferences(context);
    }

    public boolean getPrefBoolean(String key, boolean defaultValue) {
        return getDefaultSharedPreferences(context).getBoolean(key, defaultValue);
    }

    public int getPrefStringInt(String key, int defaultValue) {
        String stringPref = getDefaultSharedPreferences(context).getString(key, "" + defaultValue);

        int pref = defaultValue;
        try {
            pref = Integer.parseInt(stringPref);
        } catch (NumberFormatException ignore) {
            Log.e(BaseCaptcha.TAG, "Cannot convert property " + key + " to number, value: " + stringPref);
        }
        return pref;
    }

    public int getPrefInt(String key, int defaultValue) {
        return getDefaultSharedPreferences(context).getInt(key, defaultValue);
    }

    public long getPrefLong(String key, long defaultValue) {
        return getDefaultSharedPreferences(context).getLong(key, defaultValue);
    }

    public long getPrefStringLong(String key, long defaultValue) {
        String stringPref = getDefaultSharedPreferences(context).getString(key, ""+defaultValue);

        long pref = defaultValue;
        try {
            pref = Long.parseLong(stringPref);
        } catch (NumberFormatException ignore) {
            Log.e(BaseCaptcha.TAG, "Cannot convert property " + key + " to number, value: " + stringPref);
        }
        return pref;
    }

    public float getPrefFloat(String key, float defaultValue) {
        String stringPref = getDefaultSharedPreferences(context).getString(key, ""+defaultValue);

        float pref = defaultValue;
        try {
            pref = Float.parseFloat(stringPref);
        } catch (NumberFormatException ignore) {
            Log.e(BaseCaptcha.TAG, "Cannot convert property " + key + " to number, value: " + stringPref);
        }
        return pref;
    }

    public double getPrefDouble(String key, double defaultValue) {
        String stringPref = getDefaultSharedPreferences(context).getString(key, ""+defaultValue);

        double pref = defaultValue;
        try {
            pref = Double.parseDouble(stringPref);
        } catch (NumberFormatException ignore) {
            Log.e(BaseCaptcha.TAG, "Cannot convert property " + key + " to number, value: " + stringPref);
        }
        return pref;
    }

    public String getPrefString(String key, String defaultValue) {
        return getDefaultSharedPreferences(context).getString(key, defaultValue);
    }


    public void saveList(String key, List<String> list, String safeDelimiter) {
        Log.d(BaseCaptcha.TAG, "Saving list " + key);

        if (list == null || list.size() == 0) {
            save(key, null);
            return;
        }
        StringBuilder result = new StringBuilder();

        for (Iterator<String> iterator = list.iterator(); iterator.hasNext(); ) {
            String next = iterator.next();
            result.append(next);
            if (iterator.hasNext()) {
                result.append(safeDelimiter);
            }
        }

        Log.d(BaseCaptcha.TAG, "Saving list " + result);

        save(key, result.toString());
    }

    public void addToList(String key, String item, String safeDelimiter) {
        List<String> list = loadList(key, safeDelimiter);

        if (!list.contains(item)) {
            list.add(item);
        }

        saveList(key, list, safeDelimiter);
    }

    public void removeFromList(String key, String item, String safeDelimiter) {
        List<String> list = loadList(key, safeDelimiter);
        list.remove(item);
        saveList(key, list, safeDelimiter);
    }

    public List<String> loadList(String key, String safeDelimiter) {
        return loadList(key, safeDelimiter, null);
    }

    public List<String> loadList(String key, String safeDelimiter, String defaultList) {
        Log.d(BaseCaptcha.TAG, "Loading list " + key);

        List<String> result = new ArrayList<String>();

        String value = getPrefString(key, defaultList);

        if (value == null) {
            return result;
        }

        String[] array = value.split(safeDelimiter);
        for (String s : array) {
            if (s != null && s.length() > 0) {
                result.add(s);
            }
        }

        Log.d(BaseCaptcha.TAG, "Loading list " + result);

        return result;
    }




}
