package com.urbandroid.sleep.captchapack.flags;

import android.content.Context;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.app.ActionBar;
import android.view.Gravity;
import android.view.View;
import android.widget.ImageView;
import android.widget.Toast;

import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;

import java.io.Serializable;
import java.util.List;
import java.util.Random;

public class QuizSession implements Serializable {

    private static final int FLAGS = 3;
    public static final String KEY_QUIZ_SESSION = "session";

    private transient BaseCaptcha activity;

    private List<Flag> flags = null;
    private int rightOne = -1;
    private int progress = 1;
    private final int difficulty;
    private boolean wrongAnswerBefore = false;

    public QuizSession(final @NonNull BaseCaptcha activity) {
        this.activity = activity;
        difficulty = activity.getCaptchaSupport().getDifficulty();
    }

    public void generateNext(){
        flags = new FlagManager(activity.getApplicationContext()).take(FLAGS);
        rightOne = new Random().nextInt(FLAGS);
    }

    public void show(){

        final Context context = activity.getApplicationContext();
        int[] flagSlots = new int[] {
                R.id.flag1,
                R.id.flag2,
                R.id.flag3,
        };

        final Flag rightFlag = flags.get(rightOne);

        for (int i = 0; i < flags.size(); i++) {
            final Flag flag = flags.get(i);
            ImageView flagView = (ImageView) activity.findViewById(flagSlots[i]);
            flagView.setVisibility(View.VISIBLE);
            flagView.setImageDrawable(activity.getResources().getDrawable(flag.getResource(context)));
            flagView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    //System.out.println("----- " + flag.getName() + " "+ flag.getPopulation());
                    if (rightFlag.getIso().equals(flag.getIso())) {
                        if (progress == difficulty && !wrongAnswerBefore) {
                            activity.solved();
                            activity.finish();
                            return;
                        }
                        if (wrongAnswerBefore) {
                            wrongAnswerBefore = false;
                        } else {
                            progress++;
                        }
                        generateNext();
                        show();
                        return;
                    }
                    wrongAnswerBefore = true;

                    final Toast toast = Toast.makeText(context, "\n\t\t\t\t" + flag.getName() + "\t\t\t\t\n", Toast.LENGTH_SHORT);
                    toast.setGravity(Gravity.CENTER, 0, 0);
                    toast.show();

                }
            });
        }

        final ActionBar actionBar = activity.getSupportActionBar();
        if (actionBar != null) {
            String progressStr = difficulty == 1? "": progress + "/" + difficulty + " ";
            actionBar.setTitle(progressStr + rightFlag.getName() + "?");
        }
    }


    public void save(final @NonNull Bundle bundle){
        bundle.putSerializable(KEY_QUIZ_SESSION, this);
    }

    @Nullable
    public static QuizSession restore(final @NonNull BaseCaptcha activity, final @NonNull Bundle bundle) {
        final QuizSession session = (QuizSession) bundle.getSerializable(KEY_QUIZ_SESSION);
        if (session != null) {
            session.activity = activity;
        }
        return session;
    }

}
