package com.urbandroid.sleep.captchapack.light;

import android.content.Context;
import android.graphics.drawable.AnimationDrawable;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.Display;
import android.widget.*;
import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;

public class LightCaptcha extends BaseCaptcha implements SensorEventListener {


    private SensorManager sensorManager;

    private Sensor lightSensor;

    private int target = 15;

    private int progress = 0;

    private int threshold = 0;

    private long overThresholdTs = 0;

    private AnimationDrawable anim;

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("progress", progress);
        outState.putLong("over", overThresholdTs);
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_light);

        threshold = 30 + getCaptchaSupport().getDifficulty() * 50;
        target = getCaptchaSupport().getDifficulty() * 20000;

        sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);

        lightSensor = sensorManager.getDefaultSensor(Sensor.TYPE_LIGHT);

        if (lightSensor == null) {
            Toast.makeText(getApplicationContext(), getString(R.string.compatibility_sensor), Toast.LENGTH_LONG).show();
            solved();
            finish();
            return;
        }

        if (savedInstanceState != null) {
            progress = savedInstanceState.getInt("progress");
            overThresholdTs = savedInstanceState.getLong("over");
        }
        updateUi();

        getSupportActionBar().setTitle(R.string.hint_light);

    }

    private void animate() {
        ImageView img = (ImageView)findViewById(R.id.bg);
        img.setBackgroundDrawable(null);
        img.setBackgroundResource(R.drawable.bg_anim_light);
        anim = (AnimationDrawable) img.getBackground();
        anim.start();
    }


    @Override
    protected void onStart() {
        super.onStart();
        sensorManager.registerListener(this, lightSensor, SensorManager.SENSOR_DELAY_NORMAL);
        animate();
    }

    @Override
    protected void onStop() {
        super.onStop();
        sensorManager.unregisterListener(this, lightSensor);
        anim.stop();
    }

    private void updateUi() {

    }

    public void onSensorChanged(SensorEvent event) {

        float lux = event.values[0];

        if (lux > threshold) {
            if (overThresholdTs == -1) {
                overThresholdTs = System.currentTimeMillis();
            }
            progress = (int) (System.currentTimeMillis() - overThresholdTs);
        } else {
            overThresholdTs = -1;
            progress = 0;
        }


        if((target - progress) <= 0) {
            getCaptchaSupport().solved();
            finish();
        }

        if (overThresholdTs == -1) {
            ((ProgressBar)findViewById(R.id.progress)).setMax(threshold);
            ((ProgressBar)findViewById(R.id.progress)).setProgress((int)lux);

            Display display = getWindowManager().getDefaultDisplay();
            int width = display.getWidth();  // deprecated
            int height = display.getHeight();  // deprecated

            int dim = Math.round(Math.min(width, height) * (lux / (float) threshold));

            RelativeLayout.LayoutParams layoutParams  = new RelativeLayout.LayoutParams(dim, dim);
            layoutParams.addRule(RelativeLayout.CENTER_IN_PARENT, RelativeLayout.TRUE);
            ((ImageView)findViewById(R.id.bg)).setLayoutParams(layoutParams);

            ((TextView)findViewById(R.id.score)).setText(getString(R.string.hint_light_insufficient) + "\n" + Math.round(lux) + " LUX ");
        } else {
            ((ProgressBar)findViewById(R.id.progress)).setMax(target);
            ((ProgressBar)findViewById(R.id.progress)).setProgress(progress);

            ((TextView)findViewById(R.id.score)).setText(getString(R.string.hint_light_sufficient) + "\n" + Math.min(Math.round(100 * progress / (float) target),100)+"%");
        }

        updateUi();
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {

    }

}

