package com.urbandroid.sleep.captchapack.mirrortext;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.widget.EditText;

import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;

import java.util.Random;

public class MirrorTextCaptcha extends BaseCaptcha {

    private int solvedCounter = 0;
    private final String EXTRA_SOLVED_COUNTER = "extra_solved_counter";
    private static final String TAG = "MirrorCaptcha";
    private int difficulty = 1;
    protected String answer = "";
    protected String question = "";
    static final String QUOTE = "quote";
    private int max_len = 0;
    private int min_len = 0;
    private int nmr = 0;

    @Override
    protected void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_mirrortext);

        final MirroredTextView mirrorView = (MirroredTextView) findViewById(R.id.mirroredText);

        if (savedInstanceState != null) {
            Log.i(TAG, "Had savedInstanceState");
            question = savedInstanceState.getString(QUOTE);
        } else {
            String[] quotes = getResources().getStringArray(R.array.quotes);

            Integer quotesLen = quotes.length;
            Random r = new Random();

            nmr = r.nextInt(quotesLen);
            max_len = getLongestString(quotes).length();
            min_len = getShortestString(quotes).length();
            difficulty = getCaptchaSupport().getDifficulty();

            switch (difficulty) {
                case 1: max_len = max_len/3;
                    break;
                case 2: max_len = max_len/3;
                    break;
                case 3: max_len = 2*max_len/3;
                        min_len = max_len/3;
                    break;
                case 4: max_len = 2*max_len/3;
                        min_len = max_len/3;
                    break;
                case 5:
                        min_len = 2*max_len/3;
                    break;
            }

            while((quotes[nmr].length()< min_len) || (quotes[nmr].length()>max_len)) {
                Log.i(TAG, "nmr:" + nmr + ", len:" + quotes[nmr].length());
                nmr++;
                if (nmr >= quotesLen) {
                    nmr = 0;
                }
            }

            question = quotes[nmr];
            Log.i(TAG, "diff:" + difficulty + ", quote:" + question + ", len:" + quotes[nmr].length() + ", max min len:" + max_len + min_len);

        }
        mirrorView.setText(question);

        EditText inputQuote = (EditText) findViewById(R.id.inputQuote);

        inputQuote.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                Log.i(TAG, "max len" + max_len);
                Log.i(TAG, "question: " + question);
                Log.i(TAG, "answer: " + s.toString() + "???");
                if (checkAnswer(s.toString())) {
                    Log.i(TAG, "Solved ");
                    solved();
                    finish();
                } else {
                    alive();
                }

            }
        });
    }

    private static String getLongestString(String[] array) {
        int maxLength = 0;
        String longestString = null;
        for (String s : array) {
            if (s.length() > maxLength) {
                maxLength = s.length();
                longestString = s;
            }
        }
        return longestString;
    }

    private static String getShortestString(String[] array) {
        int minLength = getLongestString(array).length();
        String shortestString = null;
        for (String s : array) {
            if (s.length() < minLength) {
                minLength = s.length();
                shortestString = s;
            }
        }
        return shortestString;
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        savedInstanceState.putString(QUOTE, question);
        super.onSaveInstanceState(savedInstanceState);
    }

    private boolean checkAnswer(String answer) {
        return (question.equals(answer) || question.equals(answer.replace("’", "'")) || question.equals(answer.replace("'", "’")));
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        unsolved();
    }
}
