package com.urbandroid.sleep.captchapack.multi;

import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.util.Log;
import com.urbandroid.sleep.captcha.domain.CaptchaChildResult;
import com.urbandroid.sleep.captcha.util.IntentUtil;
import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;
import com.urbandroid.sleep.captchapack.context.SettingKeys;
import com.urbandroid.sleep.captchapack.context.Settings;

public abstract class BaseMultiCaptcha extends BaseCaptcha {

    private static final String EXTRA_COUNT = "extra_count";

    private int progress = 0;

    private int goal = 1;

    protected abstract int getGoal();

    public int getProgress() {
        return progress;
    }

    protected abstract boolean launchNextCaptcha();

    protected Settings settings;

    private boolean finished(final Intent intent) {
        final CaptchaChildResult result = IntentUtil.getChildResult(intent);
        Log.i(TAG, "MultiCaptcha: Back info " + result);
        if (result != null) {

            progress = new Settings(getApplicationContext()).getPrefInt(getClass() + "_" + SettingKeys.KEY_PROGRESS, 0);

            Log.i(TAG, "MultiCaptcha: Back info " + result.isSolved());

            if (result.isSolved()) {
                progress++;
                new Settings(getApplicationContext()).save(getClass() + "_" + SettingKeys.KEY_PROGRESS, progress);
                Log.i(TAG, "MultiCaptcha: Back success " + progress + " / " + goal);
                if (progress >= goal) {
                    Log.i(TAG, "MultiCaptcha: Back solved");
                    getCaptchaSupport().solved();
                    return true;
                } else {
                    if (!launchNextCaptcha()) {
                        Log.i(TAG, "MultiCaptcha: Back solved");
                        getCaptchaSupport().solved();
                        return true;
                    } else {
                        Log.i(TAG, "MultiCaptcha: Has next");
                        return false;
                    }
                }
            } else {
                Log.i(TAG, "MultiCaptcha: Back unsolved " + progress + " / " + goal);
                getCaptchaSupport().unsolved();
                return true;
            }
        } else {
            if (!launchNextCaptcha()) {
                Log.i(TAG, "MultiCaptcha: no next captcha");
                getCaptchaSupport().solved();
                return true;
            } else {
                Log.i(TAG, "MultiCaptcha: Has next");
                return false;
            }
        }
    }


    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);

        Log.i(TAG, "MultiCaptcha: onNewIntent");

        performCaptcha(intent);
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        Log.i(TAG, "MultiCaptcha: onSavedInstance");
        outState.putInt(EXTRA_COUNT, progress);

        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_captcha_multi_launch);

        Log.i(TAG, "MultiCaptcha: onCreate");

        settings = new Settings(getApplicationContext());

        goal = getGoal();

        restoreState(savedInstanceState);

        // First run of CAPTCHA reset progress
        if (getIntent() != null && progress == 0) {
            final CaptchaChildResult result = IntentUtil.getChildResult(getIntent());
            if (result == null) {
                new Settings(getApplicationContext()).save(getClass() + "_" + SettingKeys.KEY_PROGRESS, 0);
            }
        }

        performCaptcha(getIntent());

    }

    private void restoreState(Bundle savedInstanceState) {
        if (savedInstanceState != null) {
            int savedCount = savedInstanceState.getInt(EXTRA_COUNT, -1);
            if (savedCount > -1) {
                progress = savedCount;
                Log.i(TAG, "MultiCaptcha: restoring " + progress + " / " + goal);
            }
        }
    }

    private void performCaptcha(Intent intent) {
        Log.i(TAG, "MultiCaptcha: Mode operation " + getCaptchaSupport().isOperationalMode() + " preview " + getCaptchaSupport().isPreviewMode());

        if (finished(intent)) {
            finish();
            return;
        }

    }

    @Override
    protected void onDestroy() {
        Log.i(TAG, "MultiCaptcha: onDestroy ");
        super.onDestroy();
    }
}
