package com.urbandroid.sleep.captchapack.multi;


import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.drawable.Drawable;
import android.support.annotation.Nullable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.CheckedTextView;
import android.widget.ListView;

import com.urbandroid.sleep.captcha.CaptchaSupport;
import com.urbandroid.sleep.captcha.domain.CaptchaGroup;
import com.urbandroid.sleep.captcha.domain.CaptchaInfo;
import com.urbandroid.sleep.captcha.finder.CaptchaInfoFilter;
import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;
import com.urbandroid.sleep.captchapack.context.SettingKeys;
import com.urbandroid.sleep.captchapack.context.Settings;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class CaptchaAdapter extends BaseAdapter implements AdapterView.OnItemClickListener {

    private Context context;

    private List<CaptchaInfo> allCaptchas;

    private List<CaptchaInfo> selectedCaptchas = new ArrayList<>();

    private String settingsKey;

    private Settings settings;

    private ListView listView;

    public CaptchaAdapter(Context context, ListView listView, CaptchaSupport captchaSupport, String settingsKey) {
        this.context = context;
        allCaptchas = captchaSupport.getFinder().lookup(new CaptchaInfoFilter() {
            @Override
            public boolean apply(@Nullable CaptchaGroup group, @Nullable CaptchaInfo captcha) {
                if (captcha == null) {
                    return false;
                }
                if (InOrderCaptcha.class.getName().equals(captcha.getActivityName())) {
                    return false;
                }
                if (RandomCaptcha.class.getName().equals(captcha.getActivityName())) {
                    return false;
                }
                return true;
            }
        });
        this.listView = listView;

        Collections.sort(allCaptchas, new Comparator<CaptchaInfo>() {
            @Override
            public int compare(CaptchaInfo lhs, CaptchaInfo rhs) {
                return lhs.getLabel().compareTo(rhs.getLabel());
            }
        });

        this.settingsKey = settingsKey;

        settings = new Settings(listView.getContext());
        List<String> selectedIds = settings.loadList(settingsKey, SettingKeys.LIST_SEPARATOR);

        if (selectedIds.size() == 0) {
            selectedCaptchas.add(captchaSupport.getFinder().findById(1));
            settings.addToList(settingsKey, "1", SettingKeys.LIST_SEPARATOR);
            selectedCaptchas.add(captchaSupport.getFinder().findById(2));
            settings.addToList(settingsKey, "2", SettingKeys.LIST_SEPARATOR);
        }

        Log.i(BaseCaptcha.TAG, "MultiCaptchaConfig: Info " + selectedIds);

        for (String selectedId : selectedIds) {
            CaptchaInfo info = captchaSupport.getFinder().findById(Integer.parseInt(selectedId));
            if (info != null) {
                Log.i(BaseCaptcha.TAG, "MultiCaptchaConfig: Info "+info.getId());
                selectedCaptchas.add(info);
                listView.setItemChecked(allCaptchas.indexOf(info), true);
            }
        }
    }

    @Override
    public int getCount() {
        return allCaptchas.size();
    }

    @Override
    public Object getItem(int position) {
        return allCaptchas.get(position);
    }

    public CaptchaInfo getCaptchaInfo(int position) {
        return (CaptchaInfo) getItem(position);
    }

    @Override
    public long getItemId(int position) {
        return getCaptchaInfo(position).getId();
    }



    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        if (convertView == null) {
            convertView = LayoutInflater.from(parent.getContext()).inflate(R.layout.row_captcha, parent, false);
        }

        CaptchaInfo currentCaptcha = getCaptchaInfo(position);

        Log.i(BaseCaptcha.TAG, "Current "+currentCaptcha.getId() + " " + selectedCaptchas.contains(currentCaptcha));

        int order = selectedCaptchas.indexOf(currentCaptcha);

        final CheckedTextView checkedTextView = (CheckedTextView) convertView.findViewById(R.id.name);

        // TODO: icons for captcha have to be added first
        //checkedTextView.setCompoundDrawablesWithIntrinsicBounds(getActivityIcon(context, currentCaptcha), null, null, null);

        if (SettingKeys.KEY_CAPTCHA_IN_ORDER_LIST.equals(settingsKey) && order > -1) {
            checkedTextView.setText("(" + (order + 1) + ") " + currentCaptcha.getLabel());
        } else {
            checkedTextView.setText(currentCaptcha.getLabel());
        }

        listView.setItemChecked(position, selectedCaptchas.contains(currentCaptcha));
//        ((CheckedTextView)convertView.findViewById(R.id.name)).setChecked(selectedCaptchas.contains(currentCaptcha));

        return convertView;
    }

    public Drawable getActivityIcon(Context context, CaptchaInfo captchaInfo) {

        PackageManager packageManager = context.getPackageManager();

        Intent intent = new Intent();
        intent.setComponent(new ComponentName(captchaInfo.getPackageName(), captchaInfo.getActivityName()));
        ResolveInfo resolveInfo = packageManager.resolveActivity(intent, 0);

        return resolveInfo.loadIcon(packageManager);
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        CaptchaInfo clickedItem = getCaptchaInfo(position);
        if (selectedCaptchas.contains(clickedItem)) {
            selectedCaptchas.remove(clickedItem);
            settings.removeFromList(settingsKey, String.valueOf(clickedItem.getId()), SettingKeys.LIST_SEPARATOR);
        } else {
            selectedCaptchas.add(clickedItem);
            settings.addToList(settingsKey, String.valueOf(clickedItem.getId()), SettingKeys.LIST_SEPARATOR);
        }

        notifyDataSetChanged();
    }

}
