package com.urbandroid.sleep.captchapack.spinner;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.view.View;
import com.urbandroid.sleep.captchapack.R;

public class CircularProgressView extends View {

    private final int PADDING = 16;

    private Paint p = new Paint();

    private boolean[] solvedArray;
    private float azimuth;

    private int getDip(int pixel) {
        return (int) (pixel * getResources().getDisplayMetrics().density + 0.5f);
    }



    public void update(float azimuth, boolean[] solvedArray) {
        this.solvedArray = solvedArray;
        this.azimuth = azimuth;
        invalidate();
    }

    public CircularProgressView(Context context) {
        super(context);
        init();
    }

    public CircularProgressView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public CircularProgressView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init();
    }

    private void init() {
        p.setColor(getContext().getResources().getColor(R.color.colorAccent));
        p.setStyle(Paint.Style.STROKE);
        p.setStrokeWidth(getDip(3));
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        if (solvedArray == null) {
            return;
        }

        int size = Math.min(getWidth(),getHeight());

        int padding = getDip(PADDING);

        RectF innerRect = new RectF(padding, padding, size - padding, size - padding);

        for (int i = 0; i < solvedArray.length; i++) {
            boolean solved = solvedArray[i];
            if (solved) {
                canvas.drawArc(innerRect, i * SpinAroundCaptcha.SEGMENTS_ANGLE + 1, SpinAroundCaptcha.SEGMENTS_ANGLE - 1, false, p);
            }
        }

        canvas.drawArc(innerRect, azimuth - (SpinAroundCaptcha.SEGMENTS_ANGLE / 2), SpinAroundCaptcha.SEGMENTS_ANGLE, true, p);


    }


}
