package com.urbandroid.sleep.captchapack.spinner;

import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import com.urbandroid.sleep.captchapack.BaseCaptcha;
import com.urbandroid.sleep.captchapack.R;


/**
 * Created by artaud on 30.1.16.
 */
public class SpinAroundCaptcha extends BaseCaptcha implements Compass.IAzimuthListener {

    public static final int SEGMENTS_ANGLE = 10;

    private Compass compass;

    private CircularProgressView progressView;

    private int progress = 0;
    private int goal = 1;

    private boolean inverseMode = false;

    private Button leftThumb;
    private Button rightThumb;

    private boolean leftThumbDown = false;
    private boolean rightThumbDown = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_spinner);

        compass = new Compass(this);
        compass.setListener(this);
        progressView = (CircularProgressView) findViewById(R.id.progress);

        goal = getCaptchaSupport().getDifficulty();

        leftThumb = (Button) findViewById(R.id.leftThumb);
        rightThumb = (Button) findViewById(R.id.rightThumb);

        leftThumb.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getAction() == MotionEvent.ACTION_DOWN) {
                    leftThumbDown = true;
                } else if (event.getAction() == MotionEvent.ACTION_UP) {
                    leftThumbDown = false;
                }
                return false;
            }
        });

        rightThumb.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (event.getAction() == MotionEvent.ACTION_DOWN) {
                    rightThumbDown = true;
                } else if (event.getAction() == MotionEvent.ACTION_UP) {
                    rightThumbDown = false;
                }
                return false;
            }
        });


        updateUi();
    }

    private void updateUi() {
        getSupportActionBar().setTitle(getString(R.string.hint_spin, Math.max(0, goal - progress)));
        getSupportActionBar().setSubtitle(R.string.hint_spin_thumbs);
    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.d(TAG, "start compass");
        compass.start();
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.d(TAG, "stop compass");
        compass.stop();
    }

    boolean[] solvedArray = new boolean[360 / SEGMENTS_ANGLE];

    @Override
    public void onAzimuthUpdate(float azimuth) {

        if (isFinishing()) {
            return;
        }

        int index = (int) azimuth / SEGMENTS_ANGLE;

        if (leftThumbDown && rightThumbDown) {
            if (solvedArray[index] == inverseMode) {
                getCaptchaSupport().alive();
            }

            solvedArray[index] = !inverseMode;

            boolean solved = !inverseMode;

            for (int i = 0; i < solvedArray.length; i++) {
                if (solvedArray[i] == inverseMode) {
                    solved = inverseMode;
                    break;
                }
            }

            if (solved == !inverseMode) {
                progress++;
                updateUi();
                inverseMode = !inverseMode;
            }

            if (progress >= goal) {
                getCaptchaSupport().solved();
                finish();
                return;
            }

        }
        progressView.update(azimuth, solvedArray);
    }


}
